@@ -1,6 +1,6 @@
-/* Copyright (c) 1994, Joseph Arceneaux.  All rights reserved.
-
-   Copyright (c) 1992, Free Software Foundation, Inc.  All rights reserved.
+/* Copyright (c) 1999 Carlo Wood.  All rights reserved.
+   Copyright (c) 1994 Joseph Arceneaux.  All rights reserved.
+   Copyright (c) 1992 Free Software Foundation, Inc.  All rights reserved.
 
    Copyright (c) 1985 Sun Microsystems, Inc. Copyright (c) 1980 The Regents
    of the University of California. Copyright (c) 1976 Board of Trustees of
@@ -19,13 +19,17 @@
    IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
    OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. */
 
-
 /* Argument scanning and profile reading code.  Default parameters are set
    here as well. */
 
 #include "sys.h"
 #include "indent.h"
+
 #include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+RCSTAG_CC("$Id")
 
 int else_endif_col;
 
@@ -39,7 +43,6 @@ enum profile
 {
   PRO_BOOL,			/* boolean */
   PRO_INT,			/* integer */
-  PRO_FONT,			/* troff font */
   PRO_IGN,			/* ignore it */
   PRO_KEY,			/* -T switch */
   PRO_SETTINGS,			/* bundled set of settings */
@@ -64,6 +67,7 @@ static int exp_badp = 0;
 static int exp_bad = 0;
 static int exp_bap = 0;
 static int exp_bbb = 0;
+static int exp_bbo = 0;
 static int exp_bc = 0;
 static int exp_bli = 0;
 static int exp_bl = 0;
@@ -82,7 +86,6 @@ static int exp_di = 0;
 static int exp_dj = 0;
 static int exp_d = 0;
 static int exp_eei = 0;
-static int exp_ei = 0;
 static int exp_fbc = 0;
 static int exp_fbx = 0;
 static int exp_fb = 0;
@@ -92,6 +95,7 @@ static int exp_fc = 0;
 static int exp_fk = 0;
 static int exp_fs = 0;
 static int exp_gnu = 0;
+static int exp_hnl = 0;
 static int exp_orig = 0;
 static int exp_ip = 0;
 static int exp_nip = 0;
@@ -104,13 +108,12 @@ static int exp_pcs = 0;
 static int exp_pi = 0;
 static int exp_psl = 0;
 static int exp_pro = 0;
-static int exp_ps = 0;
 static int exp_kr = 0;
+static int exp_sbi = 0;
 static int exp_sc = 0;
 static int exp_sob = 0;
 static int exp_ss = 0;
 static int exp_st = 0;
-static int exp_troff = 0;
 static int exp_ts = 0;
 static int exp_v = 0;
 static int exp_version = 0;
@@ -121,18 +124,18 @@ int leave_comma;
 int decl_com_ind;
 int case_indent;
 int case_brace_indent;
+int struct_brace_indent;
 int com_ind;
 int decl_indent;
 int ljust_decl;
 int unindent_displace;
-int else_if;
 int indent_parameters;
 int ind_size;
 int tabsize;
 int blanklines_after_procs;
 int use_stdout;
-int pointer_as_binop;
 int blanklines_after_declarations;
+int blanklines_before_blockcomments;
 int blanklines_around_conditional_compilation;
 int swallow_optional_blanklines;
 int n_real_blanklines;
@@ -143,15 +146,15 @@ enum codes postfix_blankline_requested_code;
 int brace_indent;
 int btype_2;
 int braces_on_struct_decl_line;
+int break_before_boolean_operator;
+int honour_newlines;
 
 int space_sp_semicolon;
-int case_ind;
 int max_col;
 int verbose;
 int cuddle_else;
 int star_comment_cont;
 int comment_delimiter_on_blankline;
-int troff;
 int procnames_start_line;
 int proc_calls_space;
 int cast_space;
@@ -165,6 +168,7 @@ int blank_after_sizeof;
 int blanklines_after_declarations_at_proctop;
 int comment_max_col;
 int extra_expression_indent;
+int space_after_pointer_type;
 
 int expect_output_file;
 
@@ -174,9 +178,9 @@ int expect_output_file;
    last default value is the one actually assigned. */
 struct pro
 {
-  char *p_name;			/* name, eg -bl, -cli */
-  enum profile p_type;
-  int p_default;		/* the default value (if int) */
+  char *p_name;			/* option name, e.g. "bl", "cli" */
+  enum profile p_type;		/* profile type */
+  int p_default;		/* the default value (if PRO_BOOL or PRO_INT) */
 
   /* If p_type == PRO_BOOL, ON or OFF to tell how this switch affects the
      variable. Not used for other p_type's.  */
@@ -184,8 +188,8 @@ struct pro
 
   /* if p_type == PRO_SETTINGS, a (char *) pointing to a list of the switches
      to set, separated by NULs, terminated by 2 NULs. if p_type == PRO_BOOL,
-     PRO_INT, or PRO_FONT, address of the variable that gets set by the
-     option. if p_type == PRO_PRSTRING, a (char *) pointing to the string.
+     PRO_INT, address of the variable that gets set by the option.
+     if p_type == PRO_PRSTRING, a (char *) pointing to the string.
      if p_type == PRO_FUNCTION, a pointer to a function to be called. */
   int *p_obj;
 
@@ -200,100 +204,96 @@ struct pro
 /* Settings for original defaults */
 struct pro pro[] =
 {
-  {"T", PRO_KEY, 0, ONOFF_NA, 0, &exp_T},
-  {"bacc", PRO_BOOL, false, ON,
-   &blanklines_around_conditional_compilation, &exp_bacc},
-  {"badp", PRO_BOOL, false, ON,
-   &blanklines_after_declarations_at_proctop, &exp_badp},
-  {"bad", PRO_BOOL, false, ON, &blanklines_after_declarations, &exp_bad},
-  {"bap", PRO_BOOL, false, ON, &blanklines_after_procs, &exp_bap},
-  {"bc", PRO_BOOL, true, OFF, &leave_comma, &exp_bc},
-  {"bli", PRO_INT, 0, ONOFF_NA, &brace_indent, &exp_bli},
-  {"bl", PRO_BOOL, true, OFF, &btype_2, &exp_bl},
-  {"bls", PRO_BOOL, true, OFF, &braces_on_struct_decl_line, &exp_bls},
-  {"br", PRO_BOOL, true, ON, &btype_2, &exp_bl},
-  {"brs", PRO_BOOL, true, ON, &braces_on_struct_decl_line, &exp_bls},
-  {"bs", PRO_BOOL, false, ON, &blank_after_sizeof, &exp_bs},
-  {"cbi", PRO_INT, -1, ONOFF_NA, &case_brace_indent, &exp_cbi},
-  {"cdb", PRO_BOOL, true, ON, &comment_delimiter_on_blankline, &exp_cdb},
-  {"cd", PRO_INT, 33, ONOFF_NA, &decl_com_ind, &exp_cd},
-  {"ce", PRO_BOOL, true, ON, &cuddle_else, &exp_ce},
-  {"ci", PRO_INT, 4, ONOFF_NA, &continuation_indent, &exp_ci},
-  {"cli", PRO_INT, 0, ONOFF_NA, &case_indent, &exp_cli},
-  {"cp", PRO_INT, 33, ONOFF_NA, &else_endif_col, &exp_cp},
-  {"cs", PRO_BOOL, true, ON, &cast_space, &exp_cs},
-  {"c", PRO_INT, 33, ONOFF_NA, &com_ind, &exp_c},
-  {"di", PRO_INT, 16, ONOFF_NA, &decl_indent, &exp_di},
-  {"dj", PRO_BOOL, false, ON, &ljust_decl, &exp_dj},
-  {"d", PRO_INT, 0, ONOFF_NA, &unindent_displace, &exp_d},
-  {"eei", PRO_BOOL, false, ON, &extra_expression_indent, &exp_eei},
-  {"ei", PRO_BOOL, true, ON, &else_if, &exp_ei},
-  {"fbc", PRO_FONT, 0, ONOFF_NA, (int *) &blkcomf, &exp_fbc},
-  {"fbx", PRO_FONT, 0, ONOFF_NA, (int *) &boxcomf, &exp_fbx},
-  {"fb", PRO_FONT, 0, ONOFF_NA, (int *) &bodyf, &exp_fb},
-  {"fc1", PRO_BOOL, true, ON, &format_col1_comments, &exp_fc1},
-  {"fca", PRO_BOOL, true, ON, &format_comments, &exp_fca},
-  {"fc", PRO_FONT, 0, ONOFF_NA, (int *) &scomf, &exp_fc},
-  {"fk", PRO_FONT, 0, ONOFF_NA, (int *) &keywordf, &exp_fk},
-  {"fs", PRO_FONT, 0, ONOFF_NA, (int *) &stringf, &exp_fs},
-  {"gnu", PRO_SETTINGS, 0, ONOFF_NA,
-   (int *) "-nbad\0-bap\0-nbc\0-bl\0-bls\0-ncdb\0-cs\0-nce\0-di2\0-ndj\0\
--ei\0-nfc1\0-i2\0-ip5\0-lp\0-pcs\0-nps\0-psl\0-nsc\0-nsob\0-bli2\0\
--cp1\0-nfca\0", &exp_gnu},
-  {"h", PRO_FUNCTION, 0, 0, (int *) usage, &exp_version},
-  {"ip", PRO_INT, 4, ON, &indent_parameters, &exp_ip},
-  {"i", PRO_INT, 4, ONOFF_NA, &ind_size, &exp_i},
-  {"kr", PRO_SETTINGS, 0, ONOFF_NA,
-   (int *) "-nbad\0-bap\0-nbc\0-br\0-brs\0-c33\0-cd33\0-ncdb\0-ce\0\
--ci4\0-cli0\0-d0\0-di1\0-nfc1\0-i4\0-ip0\0-l75\0-lp\0-npcs\0-npsl\0-cs\0\
--nsc\0-nsc\0-nsob\0-nfca\0-cp33\0-nss\0", &exp_kr},
-  {"lc", PRO_INT, 0, ONOFF_NA, &comment_max_col, &exp_lc},
-  {"lps", PRO_BOOL, false, ON, &leave_preproc_space, &exp_lps},
-  {"lp", PRO_BOOL, true, ON, &lineup_to_parens, &exp_lp},
-  {"l", PRO_INT, 78, ONOFF_NA, &max_col, &exp_l},
-  {"nbacc", PRO_BOOL, false, OFF,
-   &blanklines_around_conditional_compilation, &exp_bacc},
-  {"nbadp", PRO_BOOL, false, OFF,
-   &blanklines_after_declarations_at_proctop, &exp_badp},
-  {"nbad", PRO_BOOL, false, OFF, &blanklines_after_declarations, &exp_bad},
-  {"nbap", PRO_BOOL, false, OFF, &blanklines_after_procs, &exp_bap},
-  {"nbc", PRO_BOOL, true, ON, &leave_comma, &exp_bc},
-  {"nbs", PRO_BOOL, false, OFF, &blank_after_sizeof, &exp_bs},
-  {"ncdb", PRO_BOOL, true, OFF, &comment_delimiter_on_blankline, &exp_cdb},
-  {"nce", PRO_BOOL, true, OFF, &cuddle_else, &exp_ce},
-  {"ncs", PRO_BOOL, true, OFF, &cast_space, &exp_cs},
-  {"ndj", PRO_BOOL, false, OFF, &ljust_decl, &exp_dj},
-  {"neei", PRO_BOOL, false, OFF, &extra_expression_indent, &exp_eei},
-  {"nei", PRO_BOOL, true, OFF, &else_if, &exp_ei},
-  {"nfc1", PRO_BOOL, true, OFF, &format_col1_comments, &exp_fc1},
-  {"nfca", PRO_BOOL, true, OFF, &format_comments, &exp_fca},
-  {"nip", PRO_SETTINGS, 0, ONOFF_NA, (int *) "-ip0", &exp_nip},
-  {"nlp", PRO_BOOL, true, OFF, &lineup_to_parens, &exp_lp},
-  {"nlps", PRO_BOOL, false, OFF, &leave_preproc_space, &exp_lps},
-  {"npcs", PRO_BOOL, false, OFF, &proc_calls_space, &exp_pcs},
-  {"npro", PRO_IGN, 0, ONOFF_NA, 0, &exp_pro},
-  {"npsl", PRO_BOOL, true, OFF, &procnames_start_line, &exp_psl},
-  {"nps", PRO_BOOL, false, OFF, &pointer_as_binop, &exp_ps},
-  {"nsc", PRO_BOOL, true, OFF, &star_comment_cont, &exp_sc},
-  {"nsob", PRO_BOOL, false, OFF, &swallow_optional_blanklines, &exp_sob},
-  {"nss", PRO_BOOL, false, OFF, &space_sp_semicolon, &exp_ss},
-  {"nv", PRO_BOOL, false, OFF, &verbose, &exp_v},
+  {"version", PRO_PRSTRING, 0, ONOFF_NA, (int *) VERSION, &exp_version},
+  {"v", PRO_BOOL, false, ON, &verbose, &exp_v},
+  {"ts", PRO_INT, 8, ONOFF_NA, &tabsize, &exp_ts},
+  {"st", PRO_BOOL, false, ON, &use_stdout, &exp_st},
+  {"ss", PRO_BOOL, false, ON, &space_sp_semicolon, &exp_ss},
+  {"sob", PRO_BOOL, false, ON, &swallow_optional_blanklines, &exp_sob},
+  {"sc", PRO_BOOL, true, ON, &star_comment_cont, &exp_sc},
+  {"sbi", PRO_INT, 0, ONOFF_NA, &struct_brace_indent, &exp_sbi},
+  {"psl", PRO_BOOL, true, ON, &procnames_start_line, &exp_psl},
+  {"pi", PRO_INT, -1, ONOFF_NA, &paren_indent, &exp_pi},
+  {"pcs", PRO_BOOL, false, ON, &proc_calls_space, &exp_pcs},
 
   {"o", PRO_BOOL, false, ON, &expect_output_file, &expect_output_file},
 
-  {"pcs", PRO_BOOL, false, ON, &proc_calls_space, &exp_pcs},
-  {"pi", PRO_INT, -1, ONOFF_NA, &paren_indent, &exp_pi},
-  {"psl", PRO_BOOL, true, ON, &procnames_start_line, &exp_psl},
-  {"ps", PRO_BOOL, false, ON, &pointer_as_binop, &exp_ps},
-  {"sc", PRO_BOOL, true, ON, &star_comment_cont, &exp_sc},
-  {"sob", PRO_BOOL, false, ON, &swallow_optional_blanklines, &exp_sob},
-  {"ss", PRO_BOOL, false, ON, &space_sp_semicolon, &exp_ss},
-  {"st", PRO_BOOL, false, ON, &use_stdout, &exp_st},
-  {"troff", PRO_BOOL, false, ON, &troff, &exp_troff},
-  {"ts", PRO_INT, 8, ONOFF_NA, &tabsize, &exp_ts},
-  {"version", PRO_PRSTRING, 0, ONOFF_NA,
-   (int *) "GNU indent " VERSION, &exp_version},
-  {"v", PRO_BOOL, false, ON, &verbose, &exp_v},
+  {"nv", PRO_BOOL, false, OFF, &verbose, &exp_v},
+  {"nss", PRO_BOOL, false, OFF, &space_sp_semicolon, &exp_ss},
+  {"nsob", PRO_BOOL, false, OFF, &swallow_optional_blanklines, &exp_sob},
+  {"nsc", PRO_BOOL, true, OFF, &star_comment_cont, &exp_sc},
+  {"npsl", PRO_BOOL, true, OFF, &procnames_start_line, &exp_psl},
+  {"npro", PRO_IGN, 0, ONOFF_NA, 0, &exp_pro},
+  {"npcs", PRO_BOOL, false, OFF, &proc_calls_space, &exp_pcs},
+  {"nlps", PRO_BOOL, false, OFF, &leave_preproc_space, &exp_lps},
+  {"nlp", PRO_BOOL, true, OFF, &lineup_to_parens, &exp_lp},
+  {"nip", PRO_SETTINGS, 0, ONOFF_NA, (int *) "-ip0", &exp_nip},
+  {"nhnl", PRO_BOOL, true, OFF, &honour_newlines, &exp_hnl},
+  {"nfca", PRO_BOOL, true, OFF, &format_comments, &exp_fca},
+  {"nfc1", PRO_BOOL, true, OFF, &format_col1_comments, &exp_fc1},
+  {"neei", PRO_BOOL, false, OFF, &extra_expression_indent, &exp_eei},
+  {"ndj", PRO_BOOL, false, OFF, &ljust_decl, &exp_dj},
+  {"ncs", PRO_BOOL, true, OFF, &cast_space, &exp_cs},
+  {"nce", PRO_BOOL, true, OFF, &cuddle_else, &exp_ce},
+  {"ncdb", PRO_BOOL, true, OFF, &comment_delimiter_on_blankline, &exp_cdb},
+  {"nbs", PRO_BOOL, false, OFF, &blank_after_sizeof, &exp_bs},
+  {"nbc", PRO_BOOL, true, ON, &leave_comma, &exp_bc},
+  {"nbbo", PRO_BOOL, true, OFF, &break_before_boolean_operator, &exp_bbo},
+  {"nbbb", PRO_BOOL, false, OFF, &blanklines_before_blockcomments, &exp_bbb},
+  {"nbap", PRO_BOOL, false, OFF, &blanklines_after_procs, &exp_bap},
+  {"nbadp", PRO_BOOL, false, OFF,
+   &blanklines_after_declarations_at_proctop, &exp_badp},
+  {"nbad", PRO_BOOL, false, OFF, &blanklines_after_declarations, &exp_bad},
+  {"nbacc", PRO_BOOL, false, OFF,
+   &blanklines_around_conditional_compilation, &exp_bacc},
+  {"lps", PRO_BOOL, false, ON, &leave_preproc_space, &exp_lps},
+  {"lp", PRO_BOOL, true, ON, &lineup_to_parens, &exp_lp},
+  {"lc", PRO_INT, DEFAULT_RIGHT_COMMENT_MARGIN,
+     ONOFF_NA, &comment_max_col, &exp_lc},
+  {"l", PRO_INT, DEFAULT_RIGHT_MARGIN, ONOFF_NA, &max_col, &exp_l},
+  {"kr", PRO_SETTINGS, 0, ONOFF_NA,
+   (int *) "-nbad\0-bap\0-nbc\0-bbo\0-hnl\0-br\0-brs\0-c33\0-cd33\0-ncdb\0\
+-ce\0-ci4\0-cli0\0-d0\0-di1\0-nfc1\0-i4\0-ip0\0-l75\0-lp\0-npcs\0-npsl\0\
+-cs\0-nsc\0-nsc\0-nsob\0-nfca\0-cp33\0-nss\0", &exp_kr},
+  {"ip", PRO_INT, 4, ONOFF_NA, &indent_parameters, &exp_ip},
+  {"i", PRO_INT, 4, ONOFF_NA, &ind_size, &exp_i},
+  {"hnl", PRO_BOOL, true, ON, &honour_newlines, &exp_hnl},
+  {"h", PRO_FUNCTION, 0, ONOFF_NA, (int *) usage, &exp_version},
+  {"gnu", PRO_SETTINGS, 0, ONOFF_NA,
+   (int *) "-nbad\0-bap\0-bbo\0-hnl\0-nbc\0-bl\0-bls\0-ncdb\0-cs\0-nce\0\
+-di2\0-ndj\0-nfc1\0-i2\0-ip5\0-lp\0-pcs\0-psl\0-nsc\0-nsob\0\
+-bli2\0-cp1\0-nfca\0", &exp_gnu},
+  {"fca", PRO_BOOL, true, ON, &format_comments, &exp_fca},
+  {"fc1", PRO_BOOL, true, ON, &format_col1_comments, &exp_fc1},
+  {"eei", PRO_BOOL, false, ON, &extra_expression_indent, &exp_eei},
+  {"dj", PRO_BOOL, false, ON, &ljust_decl, &exp_dj},
+  {"di", PRO_INT, 16, ONOFF_NA, &decl_indent, &exp_di},
+  {"d", PRO_INT, 0, ONOFF_NA, &unindent_displace, &exp_d},
+  {"cs", PRO_BOOL, true, ON, &cast_space, &exp_cs},
+  {"cp", PRO_INT, 33, ONOFF_NA, &else_endif_col, &exp_cp},
+  {"cli", PRO_INT, 0, ONOFF_NA, &case_indent, &exp_cli},
+  {"ci", PRO_INT, 4, ONOFF_NA, &continuation_indent, &exp_ci},
+  {"ce", PRO_BOOL, true, ON, &cuddle_else, &exp_ce},
+  {"cdb", PRO_BOOL, true, ON, &comment_delimiter_on_blankline, &exp_cdb},
+  {"cd", PRO_INT, 33, ONOFF_NA, &decl_com_ind, &exp_cd},
+  {"cbi", PRO_INT, -1, ONOFF_NA, &case_brace_indent, &exp_cbi},
+  {"c", PRO_INT, 33, ONOFF_NA, &com_ind, &exp_c},
+  {"bbo", PRO_BOOL, true, ON, &break_before_boolean_operator, &exp_bbo},
+  {"bs", PRO_BOOL, false, ON, &blank_after_sizeof, &exp_bs},
+  {"brs", PRO_BOOL, true, ON, &braces_on_struct_decl_line, &exp_bls},
+  {"br", PRO_BOOL, true, ON, &btype_2, &exp_bl},
+  {"bls", PRO_BOOL, true, OFF, &braces_on_struct_decl_line, &exp_bls},
+  {"bli", PRO_INT, 0, ONOFF_NA, &brace_indent, &exp_bli},
+  {"bl", PRO_BOOL, true, OFF, &btype_2, &exp_bl},
+  {"bc", PRO_BOOL, true, OFF, &leave_comma, &exp_bc},
+  {"bbb", PRO_BOOL, false, ON, &blanklines_before_blockcomments, &exp_bbb},
+  {"bap", PRO_BOOL, false, ON, &blanklines_after_procs, &exp_bap},
+  {"badp", PRO_BOOL, false, ON,
+   &blanklines_after_declarations_at_proctop, &exp_badp},
+  {"bad", PRO_BOOL, false, ON, &blanklines_after_declarations, &exp_bad},
+  {"bacc", PRO_BOOL, false, ON,
+   &blanklines_around_conditional_compilation, &exp_bacc},
+  {"T", PRO_KEY, 0, ONOFF_NA, 0, &exp_T},
 
 /* Signify end of structure.  */
   {0, PRO_IGN, 0, ONOFF_NA, 0, 0}
@@ -303,107 +303,101 @@ struct pro pro[] =
 /* Changed to make GNU style the default. */
 struct pro pro[] =
 {
-  {"T", PRO_KEY, 0, ONOFF_NA, 0, &exp_T},
-  {"bacc", PRO_BOOL, false, ON,
-   &blanklines_around_conditional_compilation, &exp_bacc},
-  {"badp", PRO_BOOL, false, ON,
-   &blanklines_after_declarations_at_proctop, &exp_badp},
-  {"bad", PRO_BOOL, false, ON, &blanklines_after_declarations, &exp_bad},
-  {"bap", PRO_BOOL, true, ON, &blanklines_after_procs, &exp_bap},
-  {"bc", PRO_BOOL, true, OFF, &leave_comma, &exp_bc},
-  {"bli", PRO_INT, 2, ONOFF_NA, &brace_indent, &exp_bli},
-  {"bl", PRO_BOOL, true, OFF, &btype_2, &exp_bl},
-  {"bls", PRO_BOOL, true, OFF, &braces_on_struct_decl_line, &exp_bls},
-  {"br", PRO_BOOL, false, ON, &btype_2, &exp_bl},
-  {"brs", PRO_BOOL, false, ON, &braces_on_struct_decl_line, &exp_bls},
-  {"bs", PRO_BOOL, false, ON, &blank_after_sizeof, &exp_bs},
-  {"cbi", PRO_INT, -1, ONOFF_NA, &case_brace_indent, &exp_cbi},
-  {"cdb", PRO_BOOL, false, ON, &comment_delimiter_on_blankline, &exp_cdb},
-  {"cd", PRO_INT, 33, ONOFF_NA, &decl_com_ind, &exp_cd},
-  {"ce", PRO_BOOL, false, ON, &cuddle_else, &exp_ce},
-  {"ci", PRO_INT, 0, ONOFF_NA, &continuation_indent, &exp_ci},
-  {"cli", PRO_INT, 0, ONOFF_NA, &case_indent, &exp_cli},
-  {"cp", PRO_INT, 1, ONOFF_NA, &else_endif_col, &exp_cp},
-  {"cs", PRO_BOOL, true, ON, &cast_space, &exp_cs},
-  {"c", PRO_INT, 33, ONOFF_NA, &com_ind, &exp_c},
-  {"di", PRO_INT, 2, ONOFF_NA, &decl_indent, &exp_di},
-  {"dj", PRO_BOOL, false, ON, &ljust_decl, &exp_dj},
-  {"d", PRO_INT, 0, ONOFF_NA, &unindent_displace, &exp_d},
-  {"eei", PRO_BOOL, false, ON, &extra_expression_indent, &exp_eei},
-  {"ei", PRO_BOOL, true, ON, &else_if, &exp_ei},
-  {"fbc", PRO_FONT, 0, ONOFF_NA, (int *) &blkcomf, &exp_fbc},
-  {"fbx", PRO_FONT, 0, ONOFF_NA, (int *) &boxcomf, &exp_fbx},
-  {"fb", PRO_FONT, 0, ONOFF_NA, (int *) &bodyf, &exp_fb},
-  {"fc1", PRO_BOOL, false, ON, &format_col1_comments, &exp_fc1},
-  {"fca", PRO_BOOL, false, ON, &format_comments, &exp_fca},
-  {"fc", PRO_FONT, 0, ONOFF_NA, (int *) &scomf, &exp_fc},
-  {"fk", PRO_FONT, 0, ONOFF_NA, (int *) &keywordf, &exp_fk},
-  {"fs", PRO_FONT, 0, ONOFF_NA, (int *) &stringf, &exp_fs},
-
-/* This is now the default. */
-  {"gnu", PRO_SETTINGS, 0, ONOFF_NA,
-   (int *) "-nbad\0-bap\0-nbc\0-bl\0-bls\0-ncdb\0-cs\0-nce\0-di2\0-ndj\0\
--ei\0-nfc1\0-i2\0-ip5\0-lp\0-pcs\0-nps\0-psl\0-nsc\0-nsob\0-bli2\0\
--cp1\0-nfca\0", &exp_gnu},
-
-  {"h", PRO_FUNCTION, 0, 0, (int *) usage, &exp_version},
-  {"ip", PRO_INT, 5, ON, &indent_parameters, &exp_ip},
-  {"i", PRO_INT, 2, ONOFF_NA, &ind_size, &exp_i},
-  {"kr", PRO_SETTINGS, 0, ONOFF_NA,
-   (int *) "-nbad\0-bap\0-nbc\0-br\0-brs\0-c33\0-cd33\0-ncdb\0-ce\0\
--ci4\0-cli0\0-d0\0-di1\0-nfc1\0-i4\0-ip0\0-l75\0-lp\0-npcs\0-npsl\0-cs\0\
--nsc\0-nsc\0-nsob\0-nfca\0-cp33\0-nss\0", &exp_kr},
-  {"lc", PRO_INT, 0, ONOFF_NA, &comment_max_col, &exp_lc},
-  {"lps", PRO_BOOL, false, ON, &leave_preproc_space, &exp_lps},
-  {"lp", PRO_BOOL, true, ON, &lineup_to_parens, &exp_lp},
-  {"l", PRO_INT, 78, ONOFF_NA, &max_col, &exp_l},
-  {"nbacc", PRO_BOOL, false, OFF,
-   &blanklines_around_conditional_compilation, &exp_bacc},
-  {"nbadp", PRO_BOOL, false, OFF,
-   &blanklines_after_declarations_at_proctop, &exp_badp},
-  {"nbad", PRO_BOOL, false, OFF, &blanklines_after_declarations, &exp_bad},
-  {"nbap", PRO_BOOL, true, OFF, &blanklines_after_procs, &exp_bap},
-  {"nbc", PRO_BOOL, true, ON, &leave_comma, &exp_bc},
-  {"nbs", PRO_BOOL, false, OFF, &blank_after_sizeof, &exp_bs},
-  {"ncdb", PRO_BOOL, false, OFF, &comment_delimiter_on_blankline, &exp_cdb},
-  {"nce", PRO_BOOL, false, OFF, &cuddle_else, &exp_ce},
-  {"ncs", PRO_BOOL, true, OFF, &cast_space, &exp_cs},
-  {"ndj", PRO_BOOL, false, OFF, &ljust_decl, &exp_dj},
-  {"neei", PRO_BOOL, false, OFF, &extra_expression_indent, &exp_eei},
-  {"nei", PRO_BOOL, true, OFF, &else_if, &exp_ei},
-  {"nfc1", PRO_BOOL, false, OFF, &format_col1_comments, &exp_fc1},
-  {"nfca", PRO_BOOL, false, OFF, &format_comments, &exp_fca},
-  {"nip", PRO_SETTINGS, 0, ONOFF_NA, (int *) "-ip0\0", &exp_nip},
-  {"nlp", PRO_BOOL, true, OFF, &lineup_to_parens, &exp_lp},
-  {"nlps", PRO_BOOL, false, OFF, &leave_preproc_space, &exp_lps},
-  {"npcs", PRO_BOOL, true, OFF, &proc_calls_space, &exp_pcs},
-  {"npro", PRO_IGN, 0, ONOFF_NA, 0, &exp_pro},
-  {"npsl", PRO_BOOL, true, OFF, &procnames_start_line, &exp_psl},
-  {"nps", PRO_BOOL, false, OFF, &pointer_as_binop, &exp_ps},
-  {"nsc", PRO_BOOL, false, OFF, &star_comment_cont, &exp_sc},
-  {"nsob", PRO_BOOL, false, OFF, &swallow_optional_blanklines, &exp_sob},
-  {"nss", PRO_BOOL, false, OFF, &space_sp_semicolon, &exp_ss},
-  {"nv", PRO_BOOL, false, OFF, &verbose, &exp_v},
+  {"version", PRO_PRSTRING, 0, ONOFF_NA, (int *) VERSION, &exp_version},
+  {"v", PRO_BOOL, false, ON, &verbose, &exp_v},
+  {"ts", PRO_INT, 8, ONOFF_NA, &tabsize, &exp_ts},
+  {"st", PRO_BOOL, false, ON, &use_stdout, &exp_st},
+  {"ss", PRO_BOOL, false, ON, &space_sp_semicolon, &exp_ss},
+  {"sob", PRO_BOOL, false, ON, &swallow_optional_blanklines, &exp_sob},
+  {"sc", PRO_BOOL, false, ON, &star_comment_cont, &exp_sc},
+  {"sbi", PRO_INT, 0, ONOFF_NA, &struct_brace_indent, &exp_sbi},
+  {"psl", PRO_BOOL, true, ON, &procnames_start_line, &exp_psl},
+  {"pi", PRO_INT, -1, ONOFF_NA, &paren_indent, &exp_pi},
+  {"pcs", PRO_BOOL, true, ON, &proc_calls_space, &exp_pcs},
   {"orig", PRO_SETTINGS, 0, ONOFF_NA,
-   (int *) "-nbap\0-nbad\0-bc\0-br\0-brs\0-c33\0-cd33\0-cdb\0-ce\0-ci4\0\
--cli0\0-cp33\0-di16\0-fc1\0-fca\0-i4\0-l75\0-lp\0\
--npcs\0-psl\0-sc\0-nsob\0-nss\0-ts8\0", &exp_orig},
+   (int *) "-nbap\0-nbad\0-bbo\0-hnl\0-bc\0-br\0-brs\0-c33\0-cd33\0-cdb\0\
+-ce\0-ci4\0-cli0\0-cp33\0-di16\0-fc1\0-fca\0-i4\0-l75\0-lp\0-npcs\0-psl\0\
+-sc\0-nsob\0-nss\0-ts8\0", &exp_orig},
 
   {"o", PRO_BOOL, false, ON, &expect_output_file, &expect_output_file},
 
-  {"pcs", PRO_BOOL, true, ON, &proc_calls_space, &exp_pcs},
-  {"pi", PRO_INT, -1, ONOFF_NA, &paren_indent, &exp_pi},
-  {"psl", PRO_BOOL, true, ON, &procnames_start_line, &exp_psl},
-  {"ps", PRO_BOOL, false, ON, &pointer_as_binop, &exp_ps},
-  {"sc", PRO_BOOL, false, ON, &star_comment_cont, &exp_sc},
-  {"sob", PRO_BOOL, false, ON, &swallow_optional_blanklines, &exp_sob},
-  {"ss", PRO_BOOL, false, ON, &space_sp_semicolon, &exp_ss},
-  {"st", PRO_BOOL, false, ON, &use_stdout, &exp_st},
-  {"troff", PRO_BOOL, false, ON, &troff, &exp_troff},
-  {"ts", PRO_INT, 8, ONOFF_NA, &tabsize, &exp_ts},
-  {"version", PRO_PRSTRING, 0, ONOFF_NA,
-   (int *) "GNU indent " VERSION, &exp_version},
-  {"v", PRO_BOOL, false, ON, &verbose, &exp_v},
+  {"nv", PRO_BOOL, false, OFF, &verbose, &exp_v},
+  {"nss", PRO_BOOL, false, OFF, &space_sp_semicolon, &exp_ss},
+  {"nsob", PRO_BOOL, false, OFF, &swallow_optional_blanklines, &exp_sob},
+  {"nsc", PRO_BOOL, false, OFF, &star_comment_cont, &exp_sc},
+  {"npsl", PRO_BOOL, true, OFF, &procnames_start_line, &exp_psl},
+  {"npro", PRO_IGN, 0, ONOFF_NA, 0, &exp_pro},
+  {"npcs", PRO_BOOL, true, OFF, &proc_calls_space, &exp_pcs},
+  {"nlps", PRO_BOOL, false, OFF, &leave_preproc_space, &exp_lps},
+  {"nlp", PRO_BOOL, true, OFF, &lineup_to_parens, &exp_lp},
+  {"nip", PRO_SETTINGS, 0, ONOFF_NA, (int *) "-ip0\0", &exp_nip},
+  {"nhnl", PRO_BOOL, true, OFF, &honour_newlines, &exp_hnl},
+  {"nfca", PRO_BOOL, false, OFF, &format_comments, &exp_fca},
+  {"nfc1", PRO_BOOL, false, OFF, &format_col1_comments, &exp_fc1},
+  {"neei", PRO_BOOL, false, OFF, &extra_expression_indent, &exp_eei},
+  {"ndj", PRO_BOOL, false, OFF, &ljust_decl, &exp_dj},
+  {"ncs", PRO_BOOL, true, OFF, &cast_space, &exp_cs},
+  {"nce", PRO_BOOL, false, OFF, &cuddle_else, &exp_ce},
+  {"ncdb", PRO_BOOL, false, OFF, &comment_delimiter_on_blankline, &exp_cdb},
+  {"nbs", PRO_BOOL, false, OFF, &blank_after_sizeof, &exp_bs},
+  {"nbc", PRO_BOOL, true, ON, &leave_comma, &exp_bc},
+  {"nbbo", PRO_BOOL, true, OFF, &break_before_boolean_operator, &exp_bbo},
+  {"nbbb", PRO_BOOL, false, OFF, &blanklines_before_blockcomments, &exp_bbb},
+  {"nbap", PRO_BOOL, true, OFF, &blanklines_after_procs, &exp_bap},
+  {"nbadp", PRO_BOOL, false, OFF,
+   &blanklines_after_declarations_at_proctop, &exp_badp},
+  {"nbad", PRO_BOOL, false, OFF, &blanklines_after_declarations, &exp_bad},
+  {"nbacc", PRO_BOOL, false, OFF,
+   &blanklines_around_conditional_compilation, &exp_bacc},
+  {"lps", PRO_BOOL, false, ON, &leave_preproc_space, &exp_lps},
+  {"lp", PRO_BOOL, true, ON, &lineup_to_parens, &exp_lp},
+  {"lc", PRO_INT, DEFAULT_RIGHT_COMMENT_MARGIN,
+     ONOFF_NA, &comment_max_col, &exp_lc},
+  {"l", PRO_INT, DEFAULT_RIGHT_MARGIN, ONOFF_NA, &max_col, &exp_l},
+  {"kr", PRO_SETTINGS, 0, ONOFF_NA,
+   (int *) "-nbad\0-bap\0-nbc\0-bbo\0-hnl\0-br\0-brs\0-c33\0-cd33\0\
+-ncdb\0-ce\0-ci4\0-cli0\0-d0\0-di1\0-nfc1\0-i4\0-ip0\0-l75\0-lp\0\
+-npcs\0-npsl\0-cs\0-nsc\0-nsob\0-nfca\0-cp33\0-nss\0", &exp_kr},
+  {"ip", PRO_INT, 5, ONOFF_NA, &indent_parameters, &exp_ip},
+  {"i", PRO_INT, 2, ONOFF_NA, &ind_size, &exp_i},
+  {"hnl", PRO_BOOL, true, ON, &honour_newlines, &exp_hnl},
+  {"h", PRO_FUNCTION, 0, ONOFF_NA, (int *) usage, &exp_version},
+/* This is now the default. */
+  {"gnu", PRO_SETTINGS, 0, ONOFF_NA,
+   (int *) "-nbad\0-bap\0-bbo\0-hnl\0-nbc\0-bl\0-bls\0-ncdb\0-cs\0\
+-nce\0-di2\0-ndj\0-nfc1\0-i2\0-ip5\0-lp\0-pcs\0-psl\0\
+-nsc\0-nsob\0-bli2\0-cp1\0-nfca\0", &exp_gnu},
+  {"fca", PRO_BOOL, false, ON, &format_comments, &exp_fca},
+  {"fc1", PRO_BOOL, false, ON, &format_col1_comments, &exp_fc1},
+  {"eei", PRO_BOOL, false, ON, &extra_expression_indent, &exp_eei},
+  {"dj", PRO_BOOL, false, ON, &ljust_decl, &exp_dj},
+  {"di", PRO_INT, 2, ONOFF_NA, &decl_indent, &exp_di},
+  {"d", PRO_INT, 0, ONOFF_NA, &unindent_displace, &exp_d},
+  {"cs", PRO_BOOL, true, ON, &cast_space, &exp_cs},
+  {"cp", PRO_INT, 1, ONOFF_NA, &else_endif_col, &exp_cp},
+  {"cli", PRO_INT, 0, ONOFF_NA, &case_indent, &exp_cli},
+  {"ci", PRO_INT, 0, ONOFF_NA, &continuation_indent, &exp_ci},
+  {"ce", PRO_BOOL, false, ON, &cuddle_else, &exp_ce},
+  {"cdb", PRO_BOOL, false, ON, &comment_delimiter_on_blankline, &exp_cdb},
+  {"cd", PRO_INT, 33, ONOFF_NA, &decl_com_ind, &exp_cd},
+  {"cbi", PRO_INT, -1, ONOFF_NA, &case_brace_indent, &exp_cbi},
+  {"c", PRO_INT, 33, ONOFF_NA, &com_ind, &exp_c},
+  {"bbo", PRO_BOOL, true, ON, &break_before_boolean_operator, &exp_bbo},
+  {"bs", PRO_BOOL, false, ON, &blank_after_sizeof, &exp_bs},
+  {"brs", PRO_BOOL, false, ON, &braces_on_struct_decl_line, &exp_bls},
+  {"br", PRO_BOOL, false, ON, &btype_2, &exp_bl},
+  {"bls", PRO_BOOL, false, OFF, &braces_on_struct_decl_line, &exp_bls},
+  {"bli", PRO_INT, 2, ONOFF_NA, &brace_indent, &exp_bli},
+  {"bl", PRO_BOOL, false, OFF, &btype_2, &exp_bl},
+  {"bc", PRO_BOOL, true, OFF, &leave_comma, &exp_bc},
+  {"bbb", PRO_BOOL, false, ON, &blanklines_before_blockcomments, &exp_bbb},
+  {"bap", PRO_BOOL, true, ON, &blanklines_after_procs, &exp_bap},
+  {"badp", PRO_BOOL, false, ON,
+   &blanklines_after_declarations_at_proctop, &exp_badp},
+  {"bad", PRO_BOOL, false, ON, &blanklines_after_declarations, &exp_bad},
+  {"bacc", PRO_BOOL, false, ON,
+   &blanklines_around_conditional_compilation, &exp_bacc},
+  {"T", PRO_KEY, 0, ONOFF_NA, 0, &exp_T},
 
 /* Signify end of structure.  */
   {0, PRO_IGN, 0, ONOFF_NA, 0, 0}
@@ -418,99 +412,94 @@ struct long_option_conversion
 
 struct long_option_conversion option_conversions[] =
 {
-  {"blank-lines-after-ifdefs", "bacc"},
-  {"blank-lines-after-procedure-declarations", "badp"},
-  {"blank-lines-after-declarations", "bad"},
-  {"blank-lines-after-procedures", "bap"},
-  {"blank-lines-after-block-comments", "bbb"},
-  {"blank-lines-after-commas", "bc"},
-  {"brace-indent", "bli"},
-  {"braces-after-if-line", "bl"},
-  {"braces-after-struct-decl-line", "bls"},
-  {"braces-on-if-line", "br"},
-  {"braces-on-struct-decl-line", "brs"},
-  {"Bill-Shannon", "bs"},
-  {"blank-before-sizeof", "bs"},
-  {"case-brace-indentation", "cbi"},
-  {"comment-delimiters-on-blank-lines", "cdb"},
-  {"declaration-comment-column", "cd"},
-  {"cuddle-else", "ce"},
-  {"continuation-indentation", "ci"},
-  {"case-indentation", "cli"},
-  {"else-endif-column", "cp"},
-  {"space-after-cast", "cs"},
-  {"comment-indentation", "c"},
-  {"declaration-indentation", "di"},
-  {"left-justify-declarations", "dj"},
-  {"line-comments-indentation", "d"},
-  {"extra-expression-indentation", "eei"},
-  {"else-if", "ei"},
-  {"*", "fbc"},
-  {"*", "fbx"},
-  {"*", "fb"},
-  {"format-first-column-comments", "fc1"},
-  {"format-all-comments", "fca"},
-  {"*", "fc"},
-  {"*", "fk"},
-  {"*", "fs"},
-  {"gnu-style", "gnu"},
-  {"help", "h"},
-  {"usage", "h"},
-  {"parameter-indentation", "ip"},
-  {"indentation-level", "i"},
-  {"indent-level", "i"},
-  {"k-and-r-style", "kr"},
-  {"kernighan-and-ritchie-style", "kr"},
-  {"kernighan-and-ritchie", "kr"},
-  {"comment-line-length", "lc"},
-  {"continue-at-parentheses", "lp"},
-  {"leave-preprocessor-space", "lps"},
-  {"remove-preprocessor-space", "nlps"},
-  {"line-length", "l"},
-  {"no-blank-lines-after-ifdefs", "nbacc"},
-  {"no-blank-lines-after-procedure-declarations", "nbadp"},
-  {"no-blank-lines-after-declarations", "nbad"},
-  {"no-blank-lines-after-procedures", "nbap"},
-  {"no-blank-lines-after-commas", "nbc"},
-  {"no-Bill-Shannon", "nbs"},
-  {"no-blank-before-sizeof", "nbs"},
-  {"no-comment-delimiters-on-blank-lines", "ncdb"},
-  {"dont-cuddle-else", "nce"},
-  {"no-space-after-casts", "ncs"},
-  {"dont-left-justify-declarations", "ndj"},
-  {"no-extra-expression-indentation", "neei"},
-  {"no-else-if", "nei"},
-  {"dont-format-first-column-comments", "nfc1"},
-  {"dont-format-comments", "nfca"},
-  {"no-parameter-indentation", "nip"},
-  {"dont-indent-parameters", "nip"},
-  {"dont-line-up-parentheses", "nlp"},
-  {"no-space-after-function-call-names", "npcs"},
-  {"ignore-profile", "npro"},
-  {"dont-break-procedure-type", "npsl"},
-  {"*", "nps"},
-  {"dont-star-comments", "nsc"},
-  {"leave-optional-blank-lines", "nsob"},
-  {"dont-space-special-semicolon", "nss"},
-  {"no-verbosity", "nv"},
-  {"output", "o"},
-  {"output-file", "o"},
-  {"original", "orig"},
-  {"original-style", "orig"},
-  {"berkeley-style", "orig"},
-  {"berkeley", "orig"},
-  {"space-after-procedure-calls", "pcs"},
-  {"paren-indentation", "pi"},
-  {"procnames-start-lines", "psl"},
-  {"pointer-as-binary-op", "ps"},
-  {"start-left-side-of-comments", "sc"},
-  {"swallow-optional-blank-lines", "sob"},
-  {"space-special-semicolon", "ss"},
-  {"standard-output", "st"},
-  {"troff-formatting", "troff"},
-  {"tab-size", "ts"},
   {"version", "version"},
   {"verbose", "v"},
+  {"usage", "h"},
+  {"tab-size", "ts"},
+  {"swallow-optional-blank-lines", "sob"},
+  {"struct-brace-indentation", "sbi"},
+  {"start-left-side-of-comments", "sc"},
+  {"standard-output", "st"},
+  {"space-special-semicolon", "ss"},
+  {"space-after-procedure-calls", "pcs"},
+  {"space-after-cast", "cs"},
+  {"remove-preprocessor-space", "nlps"},
+  {"procnames-start-lines", "psl"},
+  {"paren-indentation", "pi"},
+  {"parameter-indentation", "ip"},
+  {"output-file", "o"},
+  {"output", "o"},
+  {"original-style", "orig"},
+  {"original", "orig"},
+  {"no-verbosity", "nv"},
+  {"no-space-after-function-call-names", "npcs"},
+  {"no-space-after-casts", "ncs"},
+  {"no-parameter-indentation", "nip"},
+  {"no-extra-expression-indentation", "neei"},
+  {"no-comment-delimiters-on-blank-lines", "ncdb"},
+  {"no-blank-lines-after-procedures", "nbap"},
+  {"no-blank-lines-after-procedure-declarations", "nbadp"},
+  {"no-blank-lines-after-ifdefs", "nbacc"},
+  {"no-blank-lines-after-declarations", "nbad"},
+  {"no-blank-lines-after-commas", "nbc"},
+  {"no-blank-lines-after-block-comments", "nbbb"},
+  {"no-blank-before-sizeof", "nbs"},
+  {"no-Bill-Shannon", "nbs"},
+  {"line-length", "l"},
+  {"line-comments-indentation", "d"},
+  {"left-justify-declarations", "dj"},
+  {"leave-preprocessor-space", "lps"},
+  {"leave-optional-blank-lines", "nsob"},
+  {"kernighan-and-ritchie-style", "kr"},
+  {"kernighan-and-ritchie", "kr"},
+  {"k-and-r-style", "kr"},
+  {"indentation-level", "i"},
+  {"indent-level", "i"},
+  {"ignore-profile", "npro"},
+  {"ignore-newlines", "nhnl"},
+  {"honour-newlines", "hnl"},
+  {"help", "h"},
+  {"gnu-style", "gnu"},
+  {"format-first-column-comments", "fc1"},
+  {"format-all-comments", "fca"},
+  {"extra-expression-indentation", "eei"},
+  {"else-endif-column", "cp"},
+  {"dont-star-comments", "nsc"},
+  {"dont-space-special-semicolon", "nss"},
+  {"dont-line-up-parentheses", "nlp"},
+  {"dont-left-justify-declarations", "ndj"},
+  {"dont-indent-parameters", "nip"},
+  {"dont-format-first-column-comments", "nfc1"},
+  {"dont-format-comments", "nfca"},
+  {"dont-cuddle-else", "nce"},
+  {"dont-break-procedure-type", "npsl"},
+  {"declaration-indentation", "di"},
+  {"declaration-comment-column", "cd"},
+  {"cuddle-else", "ce"},
+  {"continue-at-parentheses", "lp"},
+  {"continuation-indentation", "ci"},
+  {"comment-line-length", "lc"},
+  {"comment-indentation", "c"},
+  {"comment-delimiters-on-blank-lines", "cdb"},
+  {"case-indentation", "cli"},
+  {"case-brace-indentation", "cbi"},
+  {"braces-on-struct-decl-line", "brs"},
+  {"braces-on-if-line", "br"},
+  {"braces-after-struct-decl-line", "bls"},
+  {"braces-after-if-line", "bl"},
+  {"brace-indent", "bli"},
+  {"break-before-boolean-operator", "bbo"},
+  {"break-after-boolean-operator", "nbbo"},
+  {"blank-lines-after-procedures", "bap"},
+  {"blank-lines-after-procedure-declarations", "badp"},
+  {"blank-lines-after-ifdefs", "bacc"},
+  {"blank-lines-after-declarations", "bad"},
+  {"blank-lines-after-commas", "bc"},
+  {"blank-lines-after-block-comments", "bbb"},
+  {"blank-before-sizeof", "bs"},
+  {"berkeley-style", "orig"},
+  {"berkeley", "orig"},
+  {"Bill-Shannon", "bs"},
   {0, 0},
 };
 
@@ -518,10 +507,10 @@ struct long_option_conversion option_conversions[] =
    argument.  Compare the two, returning true if they are equal, and if they
    are equal set *START_PARAM to point to the argument in S2.  */
 
-static int
+INLINE static int
 eqin (s1, s2, start_param)
-     register char *s1;
-     register char *s2;
+     char *s1;
+     char *s2;
      char **start_param;
 {
   while (*s1)
@@ -534,10 +523,10 @@ eqin (s1, s2, start_param)
 }
 
 /* Set the defaults. */
-void
+INLINE void
 set_defaults ()
 {
-  register struct pro *p;
+  struct pro *p;
 
   for (p = pro; p->p_name; p++)
     if (p->p_type == PRO_BOOL || p->p_type == PRO_INT)
@@ -553,12 +542,12 @@ static char *option_prefixes[] =
   0
 };
 
-static int
+INLINE static int
 option_prefix (arg)
      char *arg;
 {
-  register char **prefixes = option_prefixes;
-  register char *this_prefix, *argp;
+  char **prefixes = option_prefixes;
+  char *this_prefix, *argp;
 
   do
     {
@@ -577,6 +566,8 @@ option_prefix (arg)
   return 0;
 }
 
+extern void addkey (char *key, enum rwcodes val);
+
 /* Process an option ARG (e.g. "-l60").  PARAM is a possible value
    for ARG, if PARAM is nonzero.  EXPLICT should be nonzero iff the
    argument is being explicitly specified (as opposed to being taken from a
@@ -591,7 +582,7 @@ set_option (option, param, explicit)
 {
   struct pro *p = pro;
   char *param_start;
-  register int option_length, val;
+  int option_length, val;
 
   val = 0;
   option_length = option_prefix (option);
@@ -609,7 +600,7 @@ set_option (option, param, explicit)
       else
 	/* Long prefix */
 	{
-	  register struct long_option_conversion *o = option_conversions;
+	  struct long_option_conversion *o = option_conversions;
 	  option += option_length;
 
 	  while (o->short_name)
@@ -629,11 +620,11 @@ set_option (option, param, explicit)
     }
 
   fprintf (stderr, "indent: unknown option \"%s\"\n", option - 1);
-  exit (1);
+  exit (invocation_error);
 
 arg_missing:
   fprintf (stderr, "indent: missing argument to parameter %s\n", option);
-  exit (1);
+  exit (invocation_error);
 
 found:
   /* If the parameter has been explicitly specified, we don't
@@ -650,8 +641,10 @@ found:
 	{
 
 	case PRO_PRSTRING:
-	  puts ((char *) p->p_obj);
-	  exit (0);
+	  /* This is not really an error, but zero exit values are
+	     returned only when code has been successfully formatted. */
+	  printf ("GNU indent %s\n", (char *) p->p_obj);
+	  exit (invocation_error);
 
 	case PRO_FUNCTION:
 	  ((void(*)()) p->p_obj) ();
@@ -676,12 +669,14 @@ found:
 
 	case PRO_KEY:
 	  {
-	    register char *str;
+	    char *str;
 	    if (*param_start == 0)
-	      if (!(param_start = param))
-		goto arg_missing;
-	      else
-		val = 1;
+	      {
+		if (!(param_start = param))
+		  goto arg_missing;
+		else
+		  val = 1;
+	      }
 
 	    str = (char *) xmalloc (strlen (param_start) + 1);
 	    strcpy (str, param_start);
@@ -698,33 +693,27 @@ found:
 
 	case PRO_INT:
 	  if (*param_start == 0)
-	    if (!(param_start = param))
-	      goto arg_missing;
-	    else
-	      val = 1;
+	    {
+	      if (!(param_start = param))
+		goto arg_missing;
+	      else
+		val = 1;
+	    }
+
 	  if (!isdigit (*param_start))
 	    {
 	      fprintf (stderr,
 		     "indent: option ``%s'' requires a numeric parameter\n",
 		       option - 1);
-	      exit (1);
+	      exit (invocation_error);
 	    }
 	  *p->p_obj = atoi (param_start);
 	  break;
 
-	case PRO_FONT:
-	  if (*param_start == 0)
-	    if (!(param_start = param))
-	      goto arg_missing;
-	    else
-	      val = 1;
-	  parsefont ((struct fstate *) p->p_obj, param_start);
-	  break;
-
 	default:
 	  fprintf (stderr, "indent: set_option: internal error: p_type %d\n",
 		   (int) p->p_type);
-	  exit (1);
+	  exit (invocation_error);
 	}
     }
 
@@ -736,10 +725,10 @@ found:
 
 static void
 scan_profile (f)
-     register FILE *f;
+     FILE *f;
 {
-  register int i;
-  register char *p, *this, *next, *temp;
+  int i;
+  char *p, *this, *next, *temp;
   char b0[BUFSIZ];
   char b1[BUFSIZ];
 
@@ -747,8 +736,48 @@ scan_profile (f)
   this = 0;
   while (1)
     {
-      for (p = next; (i = getc (f)) != EOF && (*p = i) > ' '; ++p);
+      for (p = next; ((i = getc (f)) != EOF
+		      && (*p = i) > ' '
+		      && i != '/'
+		      && p < next + BUFSIZ);
+	   ++p);
 
+      if (i == '/')
+	{
+	  i = getc (f);
+	  switch (i)
+	    {
+	    case '/':
+	      do i = getc (f); while (i != EOF && i != EOL);
+	      if (i == EOF)
+		return;
+	      continue;
+
+	    case '*':
+	      do
+		{
+		  do i = getc (f); while (i != EOF && i != '*');
+		  if (i == '*')
+		    i = getc (f);
+		  if (i == EOF)
+		    {
+		      WARNING ("Profile contains unpalatable characters",
+			       0, 0);
+		      return;
+		    }
+		}
+	      while (i != '/');
+	      continue;
+
+	    default:
+	      WARNING ("Profile contains unpalatable characters", 0, 0);
+	      if (i == EOF)
+		return;
+	      continue;
+	    }
+	}
+
+      /* If we've scanned something... */
       if (p != next)
 	{
 	  *p++ = 0;
@@ -791,6 +820,7 @@ scan_profile (f)
 #define PROFILE_FORMAT "%s/%s"
 #endif
 
+extern void free ();
 
 /* set_profile looks for ./.indent.pro or $HOME/.indent.pro, in
    that order, and reads the options given in that file.  Return the
@@ -801,7 +831,7 @@ scan_profile (f)
 char *
 set_profile ()
 {
-  register FILE *f;
+  FILE *f;
   char *fname;
   static char prof[] = INDENT_PROFILE;
   char *homedir;
@@ -815,7 +845,7 @@ set_profile ()
       fname = xmalloc (len);
       fname[0] = '.';
       fname[1] = '/';
-      memcpy (&fname[2], INDENT_PROFILE, len - 3);
+      (void) memcpy (&fname[2], INDENT_PROFILE, len - 3);
       fname[len - 1] = '\0';
       return fname;
     }
